
// Copyright (C) 2002 by Luigi Pino.  All Rights Reserved.

/***************************************************************************/

#ifndef _neurosis_h_
#define _neurosis_h_

#include <windows.h>																	// Windows
#include <gl\gl.h>																		// OpenGL32
#include <gl\glu.h>																		// GLu32
#include <gl\glaux.h>																	// Glaux
#include <crtdbg.h>																		// Debug
#include <math.h>																			// Math
#include <regstr.h>																		// Registry
#include <stdio.h>																		// Standard Input/Output
#include <time.h>																			// Timer

/***************************************************************************/

// Colors
#define		R_1										1.0f
#define		G_1										1.0f
#define		B_1										1.0f
#define		R_2										0.0f
#define		G_2										0.0f
#define		B_2										0.0f
#define		R_3										0.1176f
#define		G_3										0.5608f
#define		B_3										0.9961f
#define		R_4										0.7921f
#define		G_4										0.0078f
#define		B_4										0.0078f
#define		R_5										0.9804f
#define		G_5										0.9294f
#define		B_5										0.0f
#define		R_6										0.8235f
#define		G_6										0.8392f
#define		B_6										1.0f
#define		R_7										0.8863f
#define		G_7										0.1098f
#define		B_7										0.0f
#define		R_8										0.0706f
#define		G_8										0.8431f
#define		B_8										0.0039f
#define		R_9										0.0f
#define		G_9										0.1216f
#define		B_9										0.9333f
#define		R_10									0.6275f
#define		G_10									0.6275f
#define		B_10									0.6275f
#define		R_11									0.1765f
#define		G_11									0.1843f
#define		B_11									0.5176f
#define		R_12									0.7490f
#define		G_12									0.5020f
#define		B_12									0.5020f
#define		R_13									0.5647f
#define		G_13									0.5961f
#define		B_13									0.7490f

// Textures
#define		t_font								0
#define		t_font_mask						1

/***************************************************************************/

typedef struct {
	float	r;
	float	g;
	float	b;
} color3;

/***************************************************************************/

typedef struct {
	float	x;
	float	y;
} float2;

/***************************************************************************/

typedef struct {
	float	x;
	float	y;
	float	z;
} float3;

/***************************************************************************/

typedef struct {
	int	x;
	int	y;
} int2;

/***************************************************************************/

typedef struct {
	int x;
	int	y;
	int	z;
} int3;

/***************************************************************************/

typedef struct {
	float		bottom;																			// Closest point distance to bottom of light
	float		brightness;																	// Brightness of light
	float		distance[179][360];													// Closest point distance
	float		range;																			// Range of influence
	float		top;																				// Closest point distance to top of light
	float3	position;																		// Position
}	Shadow_Sphere_Struct;

/***************************************************************************/

typedef struct {
	float3	top_upper[13];
	float3	top_lower[13];
	float3	middle[13];
	float3	bottom_upper[13];
	float3	bottom_lower[13];
}	Sphere_Struct;

/***************************************************************************/

float Cos_F(float angle);

float Cos_I(int angle);

void Fix_Angle(float *angle);

float Get_Angle_2D(float3 *from, float3 *to);

float Get_Distance(float3 *from, float3 *to);

float Get_Random(float number_amount);

void Get_Rotation_Angle(float *angle_current, float angle_dest, float *increment);

void Initialize_Angles();

void New_Length_2D(float3 *base, float3 *point, float length);

void Rotate_x(float3 *base, float3 *point, float amount);

void Rotate_y(float3 *base, float3 *point, float amount);

void Rotate_z(float3 *base, float3 *point, float amount);

float Sin_F(float angle);

float Sin_I(int angle);

/***************************************************************************/

#include "vector.h"
#include "collide.h"
#include "debug.h"
#include "input.h"
#include "camera.h"
#include "linked_lists.h"
#include "menu.h"
#include "polygon.h"
#include "window.h"
#endif

/***************************************************************************/